<?php

namespace HCM\FPA\Calculos\Apolo;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Geral\Entity\TipohierarqorgEntity;
use HCM\Geral\Entity\HierarquiaorgEntity;
use HCM\Geral\Entity\GpevinculomEntity;
use HCM\FPA\Services\CalculoFolhaService;
use HCM\FPA\Entity\FparesumoctblEntity;
use HCM\FPA\Entity\FpaProcessoEntity;
use HCM\FPA\Entity\FpaTpvincprocessoEntity;
use HCM\FPA\Entity\FpaDireitotpdemisEntity;
use HCM\FPA\Entity\FpatipocalculoEntity;
use HCM\FPA\Calculos\Geral\Inicializa;
use HCM\FPA\Calculos\Adiantamento\CalculoAdiantamento;
use HCM\FPA\Calculos\DecimoTerceiro\CalculoDecimoTerceiro;
use HCM\FPA\Calculos\Fechamento\CalculoFechamento;
use HCM\FPA\Calculos\Folha\CalculoFolha;
use HCM\FPA\Calculos\PLR\CalculoPLR;
use HCM\FPA\Calculos\Provisao\CalculoProvisao;
use HCM\FPA\Calculos\Rescisao\CalculoRescisao;
use HCM\FPA\Calculos\Geral\Dependencia;
use HCM\FPA\Calculos\Geral\MemoriaCalculo;
use HCM\FPA\Calculos\Geral\Preparacao;
use HCM\FPA\Calculos\Contabilizacao\Contabilizacao;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;
 
class PercorreCalculoConsumer{

    private $_parametros;
    private $_nrOrgPadrao;
    private $_nrOrgTrabalho;
    private $_nrEstruturaTrab;
    private $_idDebug;
    private $_ambienteSalarioOperador;
    private $_nrTipoModalidadeCalculo;
    private $_nrOcorrenciaCalculo;
    private $_mesCompetencia;
    private $_nrTipoCalculo;
    private $_simulacao;
    private $_retrocesso;
    private $_parcial;
    private $_dtPagamento;
    private $_ultPagamento;
    private $_dt_inicio;
    private $_dt_fim;
    private $_dsInicio;
    private $_dsFim;
    private $_id_retrocesso;
    private $_id_fazDifComp;
    private $_dsOrdem;
    private $_tipo_calculo;
    private $_calculo_folha;
    private $_tp_calc_mov;
    private $_vinculom;
    private $_tpVinculoProcesso;
    private $_tpRescisaoDireito;
    private $_pessoa;
    private $_dependencia;
    private $_formulario;
    private $_msg_idstatus;
    private $_msg_idabort;
    private $_msg_mensagens;
    private $_cadGozoFerias;

    const TP_TRANSF_LEGAL = '1';
    const TP_TRANSF_GEREN = '2';
    
    public function __construct($entityManager, $hcmParameters, $dataSet){
        
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->dataSet = $dataSet;
        $this->_nrTipoCalculo = $dataSet['TP_CALCULO'];
        $this->_parametros = $hcmParameters;
        $this->_nrOrgTrabalho = $this->hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao = $this->hcmParameters->getNrorgPadrao();
        $this->_nrEstruturaTrab = FPA::getEstruturaTrabalho();
        $this->_mesCompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia());
        $this->_nrTipoModalidadeCalculo = $this->hcmParameters->getTipoModalidadeCalculo();
        $this->_nrOcorrenciaCalculo = $this->hcmParameters->getNrOcorrenciaCalculo();
        $this->_buscaDadosFormulario($dataSet);
        $this->_retrocesso = false;
        $this->_parcial = false;
        $this->_cadGozoFerias = false;
        $tp_processamento = strtoupper($dataSet['TP_PROCESSAMENTO']);
        switch ($tp_processamento){
            case FPA::PROC_SIMULACAO :
                $this->_simulacao = true; break;
            case FPA::PROC_CALCULO :
                $this->_simulacao = false; break;
            case FPA::PROC_APOLO_GRAVAR :
                $this->_simulacao = false; break;
            case FPA::PROC_APOLO_GRAVAR_PARCIAL :
                $this->_simulacao = false;
                $this->_parcial = true; break;
            case FPA::PROC_RETROCESSO :
                $this->_retrocesso = true; break;
            case FPA::PROC_CALCULO_PARCIAL :
                $this->_simulacao = false;
                $this->_parcial = true; break;
            case FPA::PROC_CADGOZOFERIAS :
                $this->_cadGozoFerias = true; break; 
            default:
                throw new \Exception('Tipo de processo '.$tp_processamento.' invalido');
        }

        Inicializa::Container($entityManager, $hcmParameters, $this->_dtPagamento);
        Inicializa::carregaVariaveis();
        
        $this->_vinculom = TkDI::getContainer()->vinculom;
        $this->_tipo_calculo = TkDI::getContainer()->tipo_calculo;
        $this->_calculo_folha = TkDI::getContainer()->calculo_folha;
        $this->_pessoa = TkDI::getContainer()->pessoa;
        $this->_tp_calc_mov = TkDI::getContainer()->tp_calc_mov;
        $this->_dependencia = new Dependencia($entityManager, $hcmParameters);
        $this->_tpVinculoProcesso = new FpaTpvincprocessoEntity($entityManager, $hcmParameters);
        $this->_tpRescisaoDireito = new FpaDireitotpdemisEntity($entityManager, $hcmParameters);
        $this->objTipoCalculo = new FpatipocalculoEntity($entityManager, $hcmParameters);
        $this->_tipo_hierarquia = new TipohierarqorgEntity($entityManager, $hcmParameters);
        $this->_resumoContabil = new FparesumoctblEntity($entityManager, $hcmParameters);
        $this->verificaTipoCalculo($this->_nrTipoCalculo);
    }
    
    public function verificaTipoCalculo($nrTipoCalculo){
        
        switch ($nrTipoCalculo){
            case FpatipocalculoEntity::CALC_FOLHA:
                $this->tipoCalculo = 'HCM\FPA\Calculos\Folha\CalculoFolha';
                break;
            case FpatipocalculoEntity::CALC_FERIAS:
                $this->tipoCalculo = 'HCM\FPA\Calculos\Ferias\CalculoFerias';
                break;
            case FpatipocalculoEntity::CALC_13SAL:
                $this->tipoCalculo = 'HCM\FPA\Calculos\DecimoTerceiro\CalculoDecimoTerceiro';
                break;
            case FpatipocalculoEntity::CALC_ADIANT_13SAL:
                $this->tipoCalculo = 'HCM\FPA\Calculos\DecimoTerceiro\CalculoDecimoTerceiro';
                break;
            case FpatipocalculoEntity::CALC_ADIANTAMENTO:
                $this->tipoCalculo = 'HCM\FPA\Calculos\Adiantamento\CalculoAdiantamento';
                break;
            case FpatipocalculoEntity::CALC_RESCISAO:
                $this->tipoCalculo = 'HCM\FPA\Calculos\Rescisao\CalculoRescisao';
                break;
            case FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR:
                $this->tipoCalculo = 'HCM\FPA\Calculos\Rescisao\CalculoRescisao';
                break;
            case FpatipocalculoEntity::CALC_PROVISAO_CONTABIL:
                $this->tipoCalculo = 'HCM\FPA\Calculos\Provisao\CalculoProvisao';
                break;
            case FpatipocalculoEntity::CALC_PROVISAO_OTIMIZADA:
                $this->tipoCalculo = 'HCM\FPA\Calculos\Provisao\CalculoProvisaoOtimizada';
                break;
            case FpatipocalculoEntity::CALC_FECHAMENTO:
                $this->tipoCalculo = 'HCM\FPA\Calculos\Fechamento\CalculoFechamento';
                break;
            case FpatipocalculoEntity::CALC_ADIANT_PLR:
                $this->tipoCalculo = 'HCM\FPA\Calculos\PLR\CalculoPLR';
                return;
            case FpatipocalculoEntity::CALC_PLR:
                $this->tipoCalculo = 'HCM\FPA\Calculos\PLR\CalculoPLR';
                break;
            case FpatipocalculoEntity::CALC_CONTABILIZACAO:
            break;
            default:
                throw new \Exception('Tipo de calculo '.$nrTipoCalculo.' invalido');
        }
    }
    
    public function percorreCalculoConsumer($vinculos, $fpaCalculoConsumer) {
        $resultado = array();
        $contador = 0;
        $idstatus = MsgCalculo::ID_PENDENTE;
        $idabort = false;
        
        $objTpCalc = $this->objTipoCalculo->retornaTipoCalculo($this->_nrTipoCalculo);
        
        try {
            $this->_tp_calc_mov->retornaTipoCalculoMovimento($this->_nrTipoCalculo);
        } catch (Exception $e){
            throw $e;
        }
        
        $this->_tp_calc_mov->atribuiNrCalculoFolha(
            $this->_nrOrgTrabalho, $this->_calculo_folha, $this->_mesCompetencia, $this->_nrTipoModalidadeCalculo,
            $this->_nrOcorrenciaCalculo
        );
        
        if ($this->_nrTipoCalculo === FpatipocalculoEntity::CALC_RESCISAO || $this->_nrTipoCalculo === FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR)  {
            $this->_tpRescisaoDireito->carregaTipoDireitoPorTipoDemissao(
                $this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia
            );
        }
        
        $this->_tpVinculoProcesso->carregaTipoVinculoPorProcesso(
            $this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia
        );
        
        $calculo = new $this->tipoCalculo($this->entityManager, $this->hcmParameters, $this->dataSet, $this->_nrTipoCalculo, $this->_simulacao, $this->_parcial);
        
        $preparacao = new Preparacao($this->entityManager, $this->hcmParameters, $this->dataSet, $this->_nrTipoCalculo, false, false);
        
        $obj_tp_calc_mov = current($this->_tp_calc_mov->retornaPosicaoTpMov($this->_tp_calc_mov->retornaTpMovPrincipal()));
        $nrcalcprincipal = $obj_tp_calc_mov->getNrCalcFolha();
        
        foreach ($vinculos as $nrvinculom => $nmvinculom) {
            echo PHP_EOL."Iniciando calculo do vinculo $nrvinculom - $nmvinculom".PHP_EOL;
            
            $vinculo = $this->_vinculom->retornaVincMestreHistorico($this->_nrOrgTrabalho, $nrvinculom, $this->_mesCompetencia);
        
            $processa_msg = MsgCalculo::getInstance();
            MsgCalculo::getInstance()->setDados($nrcalcprincipal, $vinculo->getNrvinculom(), $this->_nrOrgTrabalho, 'ERROSCALCULO');
        
            if ($this->_retrocesso === false) {
                try {
                    $this->_executaPreparacao($vinculo, $preparacao, $contador);
                } catch (\Exception $e) {
                    MsgCalculo::getInstance()->adicionaMensagem($e->getMessage());
                    if (!$this->finalizaMensagens()) {
                        continue;
                    }
                    continue;
                }
            
                $this->_dependencia->analisaDependencia($vinculo);
        
                $listaProcessos = $this->_tpVinculoProcesso->getItem($vinculo->getNrtipovinculom());
                if (is_null($listaProcessos)) {
                    MsgCalculo::getInstance()->adicionaMensagem('Tipo de vínculo não possui processos de cálculo');
                    if (!$this->finalizaMensagens()) {
                        continue;
                    }
                    continue;
                }
                
                switch ($this->_nrTipoCalculo){
                    case FpatipocalculoEntity::CALC_FERIAS:
                        if (in_array(FpaProcessoEntity::FERIAS, $listaProcessos) === false){
                            continue;
                        }
                        break;
                    case FpatipocalculoEntity::CALC_13SAL:
                    case FpatipocalculoEntity::CALC_ADIANT_13SAL:
                        if (in_array(FpaProcessoEntity::DECIMO_TERCEIRO_SALARIO, $listaProcessos) === false){
                            continue;
                        }
                        break;
                    case FpatipocalculoEntity::CALC_ADIANTAMENTO:
                        if (in_array(FpaProcessoEntity::ADIANTAMENTO_SALARIO, $listaProcessos) === false){
                            continue;
                        }
                        break;
                    case FpatipocalculoEntity::CALC_RESCISAO:
                    case FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR:
                        if (in_array(FpaProcessoEntity::RESCISAO, $listaProcessos) === false){
                            continue;
                        } else {
                            $listaDireitos = $this->_tpRescisaoDireito->getItem($vinculo->getNrtpdemissao());
                            $calculo->setListaDireitos($listaDireitos);
                            if ($vinculo->getNrtpdemissao() == null){
                                MsgCalculo::getInstance()->adicionaMensagem('Vínculo sem tipo de demissão.');
                            }
                        }
                    break;
                    case FpatipocalculoEntity::CALC_PROVISAO_CONTABIL:
                        if (in_array(FpaProcessoEntity::PROVISAO, $listaProcessos) === false){
                            continue;
                        }
                        break;
                    case FpatipocalculoEntity::CALC_ADIANT_PLR:
                    case FpatipocalculoEntity::CALC_PLR:
                        if (in_array(FpaProcessoEntity::PLR, $listaProcessos) === false){
                            continue;
                        }
                        break;
                }

                try {
                    echo "Processando calculo do vinculo".PHP_EOL;
                    $calculo->setListaProcessos($listaProcessos);
                    
                    switch ($this->_nrTipoCalculo){
                        case FpatipocalculoEntity::CALC_FOLHA:
                            $calculo->calculaFolha($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                        case FpatipocalculoEntity::CALC_FERIAS:
                            $calculo->calculaFerias($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                        case FpatipocalculoEntity::CALC_13SAL:
                            $calculo->calculaDecimoTerceiro($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                        case FpatipocalculoEntity::CALC_ADIANT_13SAL:
                            $calculo->calculaDecimoTerceiro($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                        case FpatipocalculoEntity::CALC_ADIANTAMENTO:
                            $calculo->calculaAdiantamento($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                        case FpatipocalculoEntity::CALC_RESCISAO:
                            $calculo->calculaRescisao($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                        case FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR:
                            $calculo->calculaRescisao($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                        case FpatipocalculoEntity::CALC_PROVISAO_CONTABIL:
                            $calculo->calculaProvisao($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                        case FpatipocalculoEntity::CALC_PROVISAO_OTIMIZADA:
                            $calculo->calculaProvisaoOtimizada($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                        case FpatipocalculoEntity::CALC_FECHAMENTO:
                            $calculo->calculaFechamento($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                        case FpatipocalculoEntity::CALC_ADIANT_PLR:
                            $calculo->calculaPLR($vinculo, $this->_tp_calc_mov, $contador);
                            return;
                        case FpatipocalculoEntity::CALC_PLR:
                            $calculo->calculaPLR($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                    }
                    
                } catch (Exception $e) {
                    MsgCalculo::getInstance()->adicionaMensagem($e->getMessage());

                    if (!$this->finalizaMensagens()) {
                        continue;
                    }
                    continue;
                }
            } else {
                try {
                    $calculo->retroceder($vinculo, $this->_tp_calc_mov);
                } catch (Exception $e){
                    throw new \Exception($e->getMessage());
                }
            }
            if (!$this->finalizaMensagens()) {
                continue;
            }
            
            $contador++;
            $this->entityManager->getRepository(Repositories::FPA_CALCULOCONSUMER)->addNrtotprocessvinc($this->_nrOrgTrabalho, $fpaCalculoConsumer->getNrcalculoconsumer());

            echo "Finalizando calculo do vinculo $nrvinculom - $nmvinculom".PHP_EOL;
        }   

        if ($this->_msg_idabort === false) {
            try {
                $fpacalculofolha = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->findOneBy(array('nrcalculofolha' => $nrcalcprincipal, 'nrorg' => $this->_nrOrgTrabalho));
                if (($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO) or ($this->_msg_idstatus === MsgCalculo::ID_COM_ALERTA)) {
                    $fpacalculofolha->setIdstatusproc('E');
                } else {
                    $fpacalculofolha->setIdstatusproc('C');
                }
                $fpacalculofolha->persist();
                \Setforms2\Shared\Registry::getEntityManager()->flush();
            } catch (Exception $e) {
                \Setforms2\Shared\Registry::getEntityManager()->rollback();
                throw $e;
            }
        }

        \Setforms2\Shared\Registry::getEntityManager()->beginTransaction();

        $this->entityManager->getRepository(Repositories::FPA_CALCULOCONSUMER)->concluirCalculoConsumer($this->_nrOrgTrabalho, $fpaCalculoConsumer->getNrcalculoconsumer());
        $fpaCalculoConsumer->setNrcalculofolha($nrcalcprincipal);
        //$fpaCalculoConsumer->setIderro(($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO) or ($this->_msg_idstatus === MsgCalculo::ID_COM_ALERTA));
        
        \Setforms2\Shared\Registry::getEntityManager()->persist($fpaCalculoConsumer);
        \Setforms2\Shared\Registry::getEntityManager()->flush();
        \Setforms2\Shared\Registry::getEntityManager()->commit();

        echo PHP_EOL."---------------".PHP_EOL;
        echo PHP_EOL.'Numero de Calculo: '.$nrcalcprincipal.PHP_EOL;
        echo ">>>>>>>>>> Processo Finalizado <<<<<<<<<<".PHP_EOL;

        return array(
            'CALCULO' => $nrcalcprincipal,
            'MENSAGENS' => ($this->_msg_idabort === true ? $this->_msg_mensagens : null),
            'ERRO' => (($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO) or ($this->_msg_idstatus === MsgCalculo::ID_COM_ALERTA)),
            'ABORT' => $this->_msg_idabort
        );
    }
    
    public function percorreContabilizacaoConsumer($vinculos, $fpaCalculoConsumer){

        $demonstrativo = null;
        
        if ($this->_retrocesso === false) {
            if (!$this->_nrOcorrenciaCalc) {
                $listaNrCalculo = $this->_calculo_folha->carregaListaNrCalculoFolhaTodasOcorrencias(
                    $this->_nrOrgTrabalho, $this->_tipo_movimento, $this->_mesCompetencia, $this->_nrTipoModalidadeCalculo
                );

                foreach ($listaNrCalculo as $nrCalculo) {
                    $lista_calculo_folha[] = $nrCalculo;
                }
            } else {
                $arrayOcorrencias = explode(',',$this->_nrOcorrenciaCalc);
                foreach ($arrayOcorrencias as $nrocorrencia) {
                    $calculo_folha = $this->_calculo_folha->retornaCalculoFolha(
                        $this->_nrOrgTrabalho, $this->_tipo_movimento, $this->_mesCompetencia, $this->_nrTipoModalidadeCalculo, $nrocorrencia
                    );

                    foreach ($calculo_folha as $calc_folha) {
                        $lista_calculo_folha[] = $calc_folha->getNrcalculofolha();
                    }
                }
            }
        }
        
        $contabiliza = new Contabilizacao($this->entityManager, $this->hcmParameters, $this->_simulacao, $this->dataSet);
        
        $estruturasVinc = $this->_verificaEstruturasVinculos();
        
        $Associacao = $this->_verificaAssociacaoContabil($lista_calculo_folha);
        
        if($Associacao && $estruturasVinc){
            foreach ($vinculos  as $nrvinculom => $nmvinculom) {
                echo PHP_EOL."Iniciando contabilização do vinculo $nrvinculom - $nmvinculom".PHP_EOL;
                
                $vinculo = $this->_vinculom->retornaVincMestreHistorico($this->_nrOrgTrabalho, $nrvinculom, $this->_mesCompetencia);
                
                if ($this->_retrocesso === false) {
                    try {
                        echo "Processando contabilização do vinculo".PHP_EOL;
                        $result_parc = $contabiliza->contabilizar(
                            $vinculo, $lista_calculo_folha, 
                            $demonstrativo, $this->_nr_tipo_hierarquia
                        );
    
                    } catch(Exception $e) {
                        throw $e;
                    }
                } else {
                    $contabiliza->retrocedeResumoEventos($vinculo);
                }
    
                // adiciona + 1 ao contador de vinculos processados no registro do calculoconsumer
                $this->entityManager->getRepository(Repositories::FPA_CALCULOCONSUMER)->addNrtotprocessvinc($this->_nrOrgTrabalho, $fpaCalculoConsumer->getNrcalculoconsumer());
    
                echo "Finalizando contabilização do vinculo $nrvinculom - $nmvinculom".PHP_EOL;
         
                if (($this->_retrocesso === false) && ($this->_simulacao === false)) {
                    if ($this->_idFormaContab == 'C' || $this->_idFormaContab == 'T') {
                        $contabiliza->persistResumoContabil();
                    }
                    if ($this->_idFormaContab == 'D' || $this->_idFormaContab == 'T') {
                        $contabiliza->persistResumoDro();
                    }
                }
                
            }
    
            
        }else{
            $fpaCalculoConsumer->setNrtotprocessvinc($fpaCalculoConsumer->getNrtotvinculos());
        }  
        
        \Setforms2\Shared\Registry::getEntityManager()->beginTransaction();

        $this->entityManager->getRepository(Repositories::FPA_CALCULOCONSUMER)->concluirCalculoConsumer($this->_nrOrgTrabalho, $fpaCalculoConsumer->getNrcalculoconsumer());
        $fpaCalculoConsumer->setNrcalculofolha(null);
        //$fpaCalculoConsumer->setIderro(($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO) or ($this->_msg_idstatus === MsgCalculo::ID_COM_ALERTA));
        
        \Setforms2\Shared\Registry::getEntityManager()->persist($fpaCalculoConsumer);
        \Setforms2\Shared\Registry::getEntityManager()->flush();
        \Setforms2\Shared\Registry::getEntityManager()->commit();

        echo PHP_EOL."---------------".PHP_EOL;
        echo ">>>>>>>>>> Processo Finalizado <<<<<<<<<<".PHP_EOL;

        $demonstrativo = $contabiliza->preparaEmail();
        array_unshift($demonstrativo, $this->msg);
        
        return $demonstrativo;
    }
    
    private function _buscaDadosFormulario($dataSet){
        
        switch ($this->_nrTipoCalculo) {
            case FpatipocalculoEntity::CALC_FERIAS :
                $this->_buscaDadosFormularioFerias($dataSet);
                break;

            case FpatipocalculoEntity::CALC_RESCISAO :
                $this->_buscaDadosFormularioRescisao($dataSet);
                break;

            case FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR :
                $this->_buscaDadosFormularioRescocomplementar($dataSet);
                break;
            case FpatipocalculoEntity::CALC_CONTABILIZACAO :
                $this->_buscaDadosFormularioContabilizacao($dataSet);
                break;
            default:
                $this->_buscaDadosFormularioCalculo($dataSet);
                break;
        }
    }
    
    private function _buscaDadosFormularioContabilizacao($dataSet){
        $this->_formulario = $dataSet;
        $this->_dsOrdem = $dataSet['DSORDEM'];
        $this->_dsInicio = $dataSet['DSINICIO'];
        $this->_dsFim = $dataSet['DSFIM'];
        $this->_nr_tipo_hierarquia = $dataSet['NRHIERARQORG'];
        $this->_tipo_movimento = $dataSet['NRTIPOMOVIMENT'];
        $this->_nrOcorrenciaCalc = $dataSet['NROCORRECAL'];
        $this->_idFormaContab = $dataSet['IDFORMACONTAB'];
    }

    private function _buscaDadosFormularioCalculo($dataSet){
        $this->_dsOrdem = trim($dataSet['DSORDEM']);
        $this->_dsInicio = is_string($dataSet['DSINICIO']) ? trim($dataSet['DSINICIO']) : $dataSet['DSINICIO'];
        $this->_dsFim = $dataSet['DSFIM'];
        $this->_dtPagamento = $dataSet['DTPAGAMENTO'];
    }

    private function _buscaDadosFormularioFerias($dataSet){
        $this->_dsOrdem = trim($dataSet['DSORDEM']);
        $this->_dsInicio = is_string($dataSet['DSINICIO']) ? trim($dataSet['DSINICIO']) : $dataSet['DSINICIO'];
        $this->_dsFim = $dataSet['DSFIM'];
        $this->_dtPagamento = $dataSet['DTPAGAMENTO'];
        $this->_dt_inicio = DateUtil::getDataDeString(substr($dataSet['DTINIGOZOINI'],0, 10));
        $this->_dt_fim = DateUtil::getDataDeString(substr($dataSet['DTINIGOZOFIM'], 0, 10));
        $this->_dias_abono = DateUtil::getDataDeString($dataSet['QTDIASABONOFERI']);
        $this->nrFerias = $dataSet['NRFERIAS'];
    }

    private function _buscaDadosFormularioRescisao($dataSet){
        $this->_dsOrdem = trim($dataSet['DSORDEM']);
        $this->_dsInicio = is_string($dataSet['DSINICIO']) ? trim($dataSet['DSINICIO']) : $dataSet['DSINICIO'];
        $this->_dsFim = $dataSet['DSFIM'];
        $this->_dtPagamento = $dataSet['DTPAGAMENTO'];
        $this->_dt_inicio = DateUtil::getDataDeString(substr($dataSet['DTRESCISAOINI'],0, 10));
        $this->_dt_fim = DateUtil::getDataDeString(substr($dataSet['DTRESCISAOFIM'], 0, 10));
    }

    private function _buscaDadosFormularioRescocomplementar($dataSet){
        $this->_dsOrdem = trim($dataSet['DSORDEM']);
        $this->_dsInicio = is_string($dataSet['DSINICIO']) ? trim($dataSet['DSINICIO']) : $dataSet['DSINICIO'];
        $this->_dsFim = $dataSet['DSFIM'];
        $this->_dtPagamento = $dataSet['DTPAGAMENTO'];
        $this->_dt_inicio = DateUtil::getDataDeString(substr($dataSet['DTRESCISAOINI'],0, 10));
        $this->_dt_fim = DateUtil::getDataDeString(substr($dataSet['DTRESCISAOFIM'], 0, 10));
        $this->_id_fazDifComp = $dataSet['IDFAZDIFCOMP'];
    }

    private function _executaPreparacao(
         $vinculo, $preparacao, $contador
    ){
        if ($this->_nrTipoCalculo !== FpatipocalculoEntity::CALC_FOLHA &&
            $this->_nrTipoCalculo !== FpatipocalculoEntity::CALC_ADIANTAMENTO &&
            $this->_nrTipoCalculo !== FpatipocalculoEntity::CALC_FECHAMENTO
        ) {
            $preparacao->preparar($vinculo, $this->_tp_calc_mov, $contador);
        }
    }

    private function finalizaMensagens() {
        $msgcalculo = MsgCalculo::getInstance()->getDados();
        if ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ERRO) {
            $this->_msg_idstatus = MsgCalculo::ID_COM_ERRO;
        } elseif ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ALERTA and $this->_msg_idstatus != MsgCalculo::ID_COM_ERRO) {
            $this->_msg_idstatus = MsgCalculo::ID_COM_ALERTA;
        }
        if ($msgcalculo['IDABORT'] === true) {
            $this->_msg_idabort = true;
            $this->_msg_mensagens = $msgcalculo['MENSAGENS'];
            return false;
        } else {
            MsgCalculo::getInstance()->salvaMensagens();
        }
        MsgCalculo::getInstance()->clear();
        return true;
    }
    
    private function _verificaEstruturasVinculos(){

        $nrtpestrutlegal = FPA::getTipoEstruturaLegalOrg();
        $nrtpestrutgeren = FPA::getTipoEstruturaGerencialOrg();
        
        $estrutlegal = $this->_vinculom->getVinculosEstrutLegalDifereMovimentacao($this->_nrOrgTrabalho, $this->_mesCompetencia, self::TP_TRANSF_LEGAL, $nrtpestrutlegal, $this->_dsOrdem, $this->_dsInicio, $this->_dsFim);
        $estrutgeren = $this->_vinculom->getVinculosEstrutGerenDifereMovimentacao($this->_nrOrgTrabalho, $this->_mesCompetencia, self::TP_TRANSF_GEREN, $nrtpestrutgeren, $this->_dsOrdem, $this->_dsInicio, $this->_dsFim);
        
        if ($estrutlegal || $estrutgeren) {
            $this->msg .= "<b>Erros de movimentação </b><br>";
            $this->msg .= "<b>----------------------</b><br>";

            if($estrutlegal) {
                $this->msg .= "<b>Vinculos com estrutura legal do histórico diferente da movimentação legal: </b><br>";
                /** @var GpevinculomEntity $vinculo */
                foreach ($estrutlegal as $vinculo) {
                    $this->msg .= $vinculo->getNrvinculom() . " - " . $vinculo->getNmvinculom() . "<br>";
                }
            }

            if($estrutgeren) {
                $this->msg .= "<b>Vinculos com estrutura gerencial do histórico diferente da movimentação gerencial: </b><br>";
                /** @var GpevinculomEntity $vinculo */
                foreach ($estrutgeren as $vinculo) {
                    $this->msg .= $vinculo->getNrvinculom() . " - " . $vinculo->getNmvinculom() . "<br>";
                }
            }
            return false;
        }
        return true;
    }

    private function _verificaAssociacaoContabil($lista_calculo_folha){

        $erros = array();

        foreach ($lista_calculo_folha as $nr_calculo_folha) {
            $eventos = $this->_resumoContabil->getEventoSemAssociacaoPorSelecaoVinculos($this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia, $nr_calculo_folha, $this->_dsOrdem, $this->_dsInicio, $this->_dsFim);

            if ($eventos) {
               $erros = array_merge($erros, $eventos);
            }
        }
        if (!empty($erros)) {
            $this->msg .= "<b>Erros - Eventos sem  Associação Contábil para a estrutura legal</b><br>";
            $this->msg .= "<b>-----------------------------</b><br>";

            foreach ($erros as $evento) {
                $this->msg .= $evento['nreventom'] . " - " . $evento['nmeventoh'] . "<br>";
            }
            return false;
        }
        return true;
    }
}