<?php

namespace HCM\FPA\Entity;

use HCM\Util\Repositories;
use HCM\Util\TkEntity;
use Zeedhi\Framework\ORM\DateTime;
use HCM\Util\DateUtil;
use Teknisa\Model\Gestaopessoas\Repositories\FpaResumodro;

class FparesumodroEntity extends TkEntity
{

    public function __construct($entityManager, $hcmParameters) {
        $this->hcmParameters = $hcmParameters;
        $this->fpa_resumoDro = Repositories::FPA_RESUMODRO;
        parent::__construct(null,$entityManager);
    }

    public function retornaResumoDroPorEstruturaMesCompetencia($nr_org, $dt_ocorrencia, $nr_estrutura, $nr_tipomoviment){
        $resultado = $this->entityManager->getRepository(Repositories::FPA_RESUMODRO)->retornaResumoDroPorEstruturaMesCompetencia(
            $nr_org, $dt_ocorrencia, $nr_estrutura, $nr_tipomoviment
        );
        return $resultado;
    }

    public function retornaResumoDroPorEstruturaMesCompetenciaMovimento($nr_org, $dt_ocorrencia, $nr_tipomoviment)
    {
        $resultado = $this->entityManager->getRepository(Repositories::FPA_RESUMODRO)->retornaResumoDroPorEstruturaMesCompetenciaMovimento(
            $nr_org, $dt_ocorrencia, $nr_tipomoviment
        );
        return $resultado;
    }

    public function retornaResumoDroPorEstruturaSupMesCompetencia($nr_org, $dt_ocorrencia, $nr_estrutura_sup, $nr_tipomoviment){
        $resultado = $this->entityManager->getRepository(Repositories::FPA_RESUMODRO)->retornaResumoDroPorEstruturaSupMesCompetencia(
            $nr_org, $dt_ocorrencia, $nr_estrutura_sup, $nr_tipomoviment
        );
        return $resultado;
    }

    public function retornaResumoDroPorEstruturasMesCompetencia($nr_org, $dt_ocorrencia, $nr_estrutura_sup, $nr_estrutura, $nr_tipomoviment){
        $resultado = $this->entityManager->getRepository(Repositories::FPA_RESUMODRO)->retornaResumoDroPorEstruturasMesCompetencia(
            $nr_org, $dt_ocorrencia, $nr_estrutura_sup, $nr_estrutura, $nr_tipomoviment
        );
        return $resultado;
    }

    public function criaListaResumoDro($nr_org, $dt_ocorrencia, $nr_estrutura, $nr_tipomoviment){
        $resumo = $this->retornaResumoDroPorEstruturaMesCompetencia(
            $nr_org, $dt_ocorrencia, $nr_estrutura, $nr_tipomoviment
        );
        
        foreach ($resumo as $item) {
            $this->addItem($item, $item->getNrresumodro());
        }

        if (is_array($resumo) === true) {
            return true;
        } else {
            return false;
        }
    }

    public function criaListaResumoDroLite($nr_org, $dt_ocorrencia, $nr_tipomoviment){
        $resumo = $this->retornaResumoDroPorEstruturaMesCompetenciaMovimento(
            $nr_org, $dt_ocorrencia, $nr_tipomoviment
        );

        foreach ($resumo as $item) {
            $this->addItem($item, $item->getNrresumodro());
        }

        if (is_array($resumo) === true) {
            return true;
        } else {
            return false;
        }
    }
    
    public function criaListaResumoDroPorEstruturaSup($nr_org, $dt_ocorrencia, $nr_estrutura_sup, $nr_tipomoviment){
        $resumo = $this->retornaResumoDroPorEstruturaSupMesCompetencia(
            $nr_org, $dt_ocorrencia, $nr_estrutura_sup, $nr_tipomoviment
        );
        foreach ($resumo as $item) {
            $this->addItem($item, $item->getNrresumodro());
        }

        if (is_array($resumo) === true) {
            return true;
        } else {
            return false;
        }
    }
    
    public function criaListaResumoDroPorEstruturas($nr_org, $dt_ocorrencia, $nr_estrutura_sup, $nr_estrutura, $nr_tipomoviment){
        $resumo = $this->retornaResumoDroPorEstruturasMesCompetencia(
            $nr_org, $dt_ocorrencia, $nr_estrutura_sup, $nr_estrutura, $nr_tipomoviment
        );
        foreach ($resumo as $item) {
            $this->addItem($item, $item->getNrresumodro());
        }

        if (is_array($resumo) === true) {
            return true;
        } else {
            return false;
        }
    }
    
    public function procuraResumoDroPorDespesa($despesa, $nrEstrutura){
        $itens = $this->getItems();
        foreach ($itens as $item) {
            if ( ($despesa === $item->getCddespadmfat())
               && ($nrEstrutura === $item->getNrestruturam())
            ) {
                return $item;
            }
        }
    }

    public function atualizaListaResumoDro(
        $nrorg, $dt_ocorrencia, $nr_estruturam, $despesa, $valor_evento, $nr_estrutura_sup = null,
        $nr_tipomoviment = null, $nr_qtde_vinc = 0
    ){
        
        $dt_ocorrencia = DateUtil::getDataDeString($dt_ocorrencia, DateUtil::FORMATO_BRASILEIRO, true);
        $item = $this->procuraResumoDroPorDespesa($despesa, $nr_estruturam);

        if (is_object($item) === true) {
            $valor_lancamento = $item->getVrlancamento() + $valor_evento;
            $qtde_vinc = $item->getNrqtdevinc() + $nr_qtde_vinc;
            $item->setNrorg($item->getNrorg());
            $item->setCddespadmfat($despesa);
            $item->setVrlancamento($valor_lancamento);
            $item->setNrqtdevinc($qtde_vinc);
            $item->setDtultatu(DateUtil::getDataAtual());
            $item->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $item->setCdoperultatu($this->hcmParameters->getCdoperador());
        } else {
            $item = new $this->fpa_resumoDro;
            $item->setNrorg($nrorg);
            $item->setDtocorrencia($dt_ocorrencia);
            $item->setNrtipomoviment($nr_tipomoviment);
            $item->setNrestruturamsup($nr_estrutura_sup);
            $item->setNrestruturam($nr_estruturam);
            $item->setCddespadmfat($despesa);
            $item->setVrlancamento($valor_evento);
            $item->setNrqtdevinc($nr_qtde_vinc);
            
            $item->setDtinclusao(DateUtil::getDataAtual());
            $item->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $item->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $item->setDtultatu(DateUtil::getDataAtual());
            $item->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $item->setCdoperultatu($this->hcmParameters->getCdoperador());

            $item->setNrresumodro(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RESUMODRO', 1, 12, true));

            $this->addItem($item, $item->getNrresumodro());
        }
    }

    public function retrocedeListaResumoDro(){
        $itens = $this->getItems();
        if($itens){
            foreach ($itens as $item) {
                $this->removeItem($item);
            }
            return true;
        }else{
            return false;
        }    
    }
    
    public function removeItensZerados(){
        $itens = $this->getItems();
        if($itens){
            foreach ($itens as $item) {
                if($item->getVrlancamento()==0)
                    $this->removeItem($item);
            }
            return true;
        }else{
            return false;
        }    
    }

}