<?php

namespace HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\santander\registro;

use HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\RegistroHolerite;

use HCM\Util\DateUtil;

class Registro3HoleriteSantander extends RegistroHolerite{

    const REGISTRO_3 = '3';
    const BASE_IRRF = 'BASE IRRF';
    const BASE_INSS = 'BASE INSS';
    const BASE_FGTS = 'BASE FGTS';
    const FGTS_MES = 'FGTS MES';

    private $_pessoa;
    private $_vinculo;
    private $_dadosVinculo;


    public function geraRegistro3(){
        $this->_geraRegistro();
    }


    private function _geraRegistro(){
        // Bloco 01 | 0001-0001 | Tipo de registro = 3.
        $this->atribuiDado(self::REGISTRO_3, 1);
        // Bloco 02 | 0002-0021 | Matrícula do funcionário.
        $this->atribuiDado($this->_vinculo->getCdmatricula(), 20, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 03 | 0022-0036 | Total Valor a Débito.
        $this->atribuiDado(sprintf('%1.2f', array_key_exists(self::REF_TOTAL_DEBITO,$this->_dadosVinculo)?$this->_dadosVinculo[self::REF_TOTAL_DEBITO]:""), 15, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 04 | 0037-0051 | Total Valor a Crédito.
        $this->atribuiDado(sprintf('%1.2f', (array_key_exists(self::REF_TOTAL_CREDITO,$this->_dadosVinculo))?($this->_dadosVinculo[self::REF_TOTAL_CREDITO]):""), 15, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 05 | 0052-0066 | Total Valor Líquido.
        $this->atribuiDado(sprintf('%1.2f', (array_key_exists(self::REF_TOTAL_LIQUIDO,$this->_dadosVinculo))?($this->_dadosVinculo[self::REF_TOTAL_LIQUIDO]):""), 15, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 06 | 0067-0076 | Quantidade de Lançamentos.'
        $this->atribuiDadoEspecial((string)(array_key_exists(self::REF_QUANT_LANC,$this->_dadosVinculo))?($this->_dadosVinculo[self::REF_QUANT_LANC]):"", 10, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 07 | 0077-0085 | Base IRRF.
        $this->atribuiDado(self::BASE_IRRF, 9, self::CARACTER_BRANCO, self::PREENCHE_DIREITA);

        // Bloco 08 | 0086-0095 | Valor Base IRRF.
        $this->atribuiDado((array_key_exists(self::REF_BASE_IRRF,$this->_dadosVinculo))?($this->_dadosVinculo[self::REF_BASE_IRRF]):"", 10, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 09 | 0096-0104 | Uso Exclusivo do Banco (Preencher com brancos).
        $this->atribuiDado(self::CARACTER_BRANCO, 9, self::CARACTER_BRANCO, self::PREENCHE_DIREITA);

        // Bloco 10 | 0105-0113 | Base INSS.
        $this->atribuiDado(self::BASE_INSS, 9, self::CARACTER_BRANCO, self::PREENCHE_DIREITA);

        // Bloco 11 | 0114-0123 | Valor Base INSS.
        $this->atribuiDado((array_key_exists(self::REF_BASE_INSS,$this->_dadosVinculo))?($this->_dadosVinculo[self::REF_BASE_INSS]):"", 10, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 12 | 0124-0132 | Base FGTS.
        $this->atribuiDado(self::BASE_FGTS, 9, self::CARACTER_BRANCO, self::PREENCHE_DIREITA);

        // Bloco 13 | 0133-0142 | Valor Base FGTS.
        $this->atribuiDado((array_key_exists(self::REF_BASE_FGTS,$this->_dadosVinculo))?($this->_dadosVinculo[self::REF_BASE_FGTS]):"", 10, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 14 | 0143-0151 | Uso Exclusivo do Banco (Preencher com brancos).
        $this->atribuiDado(self::CARACTER_BRANCO, 9, self::CARACTER_BRANCO, self::PREENCHE_DIREITA);

        // Bloco 15 | 0152-0160 | FGTS Mês.
        $this->atribuiDado(self::FGTS_MES, 9, self::CARACTER_BRANCO, self::PREENCHE_DIREITA);

        // Bloco 16 | 0161-0170 | Valor FGTS Mês.
        $this->atribuiDado((array_key_exists(self::REF_FGTS_MES,$this->_dadosVinculo))?($this->_dadosVinculo[self::REF_FGTS_MES]):"", 10, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 17 | 0171-0400 | Livre (Preencher com espaços).
        $this->atribuiDado(self::CARACTER_BRANCO, 230, self::CARACTER_BRANCO, self::PREENCHE_DIREITA);
    }


    public function setPessoa($pessoa)
    {
        $this->_pessoa = $pessoa;
    }


    public function setVinculo($vinculo)
    {
        $this->_vinculo = $vinculo;
    }


    public function setDadosVinculo($dados)
    {
        $this->_dadosVinculo = $dados;
    }


}